@@ -6,7 +6,7 @@
 #include <locale.h>
 #include <glib/gi18n.h>
 
-static void
+G_GNUC_PRINTF(1, 2) static void
 gcab_error (const gchar *format, ...)
 {
     va_list args;
@@ -66,6 +66,27 @@ remove_leading_path (gchar *name)
     return name + i;
 }
 
+static
+void save_array_to_file (const GByteArray *array, const gchar *base, const gchar *suffix)
+{
+    GError *error = NULL;
+    gchar *resname = g_strdup_printf ("%s.%s", base, suffix);
+    g_print (_("Dumping %s data to: %s ...\n"), suffix, resname);
+    GFile *outputfile = g_file_new_for_commandline_arg (resname);
+    GOutputStream *output = G_OUTPUT_STREAM (g_file_replace (outputfile, NULL, FALSE, 0, NULL, &error));
+
+    if (!error)
+        g_output_stream_write_all (output, array->data, array->len, NULL, NULL, &error);
+
+    if (error)
+        gcab_error (_("can't write file %s: %s"), resname, error->message);
+
+    g_object_unref (output);
+    g_object_unref (outputfile);
+    g_free (resname);
+    g_clear_error (&error);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -77,19 +98,23 @@ main (int argc, char *argv[])
     gchar *change = NULL;
     int version = 0;
     int nopath = 0;
+    int space = 0;
     int compress = 0;
     int list = 0;
     int create = 0;
     int extract = 0;
+    int dump_reserved = 0;
     GOptionEntry entries[] = {
         { "version", 0, 0, G_OPTION_ARG_NONE, &version, N_("Print program version"), NULL },
         { "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose, N_("Be verbose"), NULL },
         { "create", 'c', 0, G_OPTION_ARG_NONE, &create, N_("Create archive"), NULL },
         { "extract", 'x', 0, G_OPTION_ARG_NONE, &extract, N_("Extract all files"), NULL },
+        { "dump-reserved", 'D', 0, G_OPTION_ARG_NONE, &dump_reserved, N_("Dump reserved and extra data"), NULL },
         { "list", 't', 0, G_OPTION_ARG_NONE, &list, N_("List content"), NULL },
         { "directory", 'C', 0, G_OPTION_ARG_FILENAME, &change, N_("Change to directory DIR"), N_("DIR") },
         { "zip", 'z', 0, G_OPTION_ARG_NONE, &compress, N_("Use zip compression"), NULL },
         { "nopath", 'n', 0, G_OPTION_ARG_NONE, &nopath, N_("Do not include path"), NULL },
+        { "space", 's', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_INT, &space, N_("Reserve space in cabinet for signing (e.g. -s 6144 reserves 6K bytes)"), NULL },
         { G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &args, NULL, N_("FILE INPUT_FILES...") },
         { NULL }
     };
@@ -123,7 +148,7 @@ individual files from the archive.\
         return 0;
     }
 
-    if ((list + extract + create) != 1)
+    if ((list + extract + create + dump_reserved) != 1)
         gcab_error (_("Please specify a single operation."));
 
     if (!args || args[0] == NULL)
@@ -136,7 +161,7 @@ individual files from the archive.\
     GOutputStream *output;
     GFile *cwd;
 
-    if (list || extract) {
+    if (list || extract || dump_reserved) {
         GFile *file = g_file_new_for_commandline_arg (args[0]);
         GInputStream *in = G_INPUT_STREAM (g_file_read (file, cancellable, &error));
 
@@ -161,6 +186,20 @@ individual files from the archive.\
 
             if (!gcab_cabinet_extract (cabinet, file, file_callback, NULL, NULL, cancellable, &error))
                 gcab_error (_("error during extraction: %s"), error->message);
+        } else if (dump_reserved) {
+            GByteArray *reserved;
+
+            g_object_get (cabinet, "reserved", &reserved, NULL);
+            if (reserved != NULL) {
+                save_array_to_file (reserved, args[0], "header");
+                g_byte_array_unref (reserved);
+            }
+
+            reserved = (GByteArray *)gcab_cabinet_get_signature (cabinet, cancellable, &error);
+            if (error)
+                gcab_error (_("error while reading signature: %s"), error->message);
+            if (reserved != NULL)
+                save_array_to_file (reserved, args[0], "signature");
         }
 
         g_object_unref (in);
@@ -171,6 +210,13 @@ individual files from the archive.\
     if (args[1] == NULL)
         gcab_error (_("please specify input files."));
 
+    if (space) {
+        GByteArray *reserved = g_byte_array_sized_new (space);
+        g_byte_array_set_size (reserved, space);
+        g_object_set (cabinet, "reserved", reserved, NULL);
+        g_byte_array_unref (reserved);
+    }
+
     folder = gcab_folder_new (compress ? GCAB_COMPRESSION_MSZIP : 0);
 
     for (i = 1; args[i]; i++) {
@@ -200,7 +246,7 @@ individual files from the archive.\
 
     cwd = g_file_new_for_commandline_arg (".");
     if (!gcab_cabinet_add_folder (cabinet, folder, &error))
-        gcab_error (_("can't add folder to cabinet: %s"), args[0], error->message);
+        gcab_error (_("can't add folder to cab file %s: %s"), args[0], error->message);
 
     if (!gcab_cabinet_write (cabinet, output,
                              file_callback,
