@@ -13,7 +13,9 @@
 #include <unistd.h>
 #include <time.h>
 #include <zlib.h>
+
 #include "gcab-folder.h"
+#include "decomp.h"
 
 /* based on the spec
    http://msdn.microsoft.com/en-us/library/bb417343.aspx */
@@ -94,9 +96,13 @@ struct cdata
     u2 ncbytes;
     u2 nubytes;
     guint8 *reserved;
-    guint8 data[DATABLOCKSIZE];
-    guint8 out[DATABLOCKSIZE];
+    guint8 in[CAB_INPUTMAX+2];
+    guint8 out[CAB_BLOCKMAX];
+    /* using zlib */
     z_stream z;
+    /* using wine decomp.h */
+    FDI_Int fdi;
+    fdi_decomp_state decomp;
 };
 
 gboolean     cheader_write                      (cheader_t *ch,
@@ -134,7 +140,7 @@ gboolean     cdata_write                        (cdata_t *cd,
                                                  GError **error);
 gboolean     cdata_read                         (cdata_t *cd,
                                                  u1 res_data,
-                                                 GCabCompression compression,
+                                                 gint comptype,
                                                  GDataInputStream *in,
                                                  GCancellable *cancellable,
                                                  GError **error);
