@@ -32,6 +32,9 @@ gcab_compression_get_type (void)
       static const GEnumValue values[] = {
         { GCAB_COMPRESSION_NONE, "GCAB_COMPRESSION_NONE", "none" },
         { GCAB_COMPRESSION_MSZIP, "GCAB_COMPRESSION_MSZIP", "mszip" },
+        { GCAB_COMPRESSION_QUANTUM, "GCAB_COMPRESSION_QUANTUM", "quantum" },
+        { GCAB_COMPRESSION_LZX, "GCAB_COMPRESSION_LZX", "lzx" },
+        { GCAB_COMPRESSION_MASK, "GCAB_COMPRESSION_MASK", "mask" },
         { 0, NULL, NULL }
       };
       GType g_define_type_id =
